EPOC.UI.HTMLView = function(container, data) {
	EPOC.UI.View.apply(this, arguments);
	this.popup = document.createElement("div");
	EPOC.Dom.addCSSClass(this.popup, "modal");
	this.popup.id = "popup";
	this.popup.style = "display:block";
	this.popupContent = document.createElement("p");
	this.popup.appendChild(this.popupContent);
	this.mask = document.createElement("div");
	EPOC.Dom.addCSSClass(this.mask, "mask");
	this.mask.id = "mask";
	this.mask.style.display = 'none';
	this.popup.style.display = 'none';
	this.container.appendChild(this.mask);
	this.container.appendChild(this.popup);
    this.lastWidth = 0;
    this.lastHeight = 0;
};

EPOC.UI.HTMLView.extend(EPOC.UI.View, {
	dummyHref : "javascript:void(0);",

	scrollToNode : function(node) {
		var node = EPOC.Dom.get(node);
		if (node) {
			this.container.scrollTop = node.offsetY + 32;
		}
	},

	draw : function() {
		this.container.innerHTML = this.data;
		this.makeLinksTabbable(this.container);
	},

	// recursively look for A tags and buttons to
	// allow the user to be able to tab to them
	makeLinksTabbable : function(node) {
		if (!node.childNodes)
			return;

		var child = node.childNodes;

		for ( var n = 0; n < child.length; n++) {
			if (child[n].tagName == 'A') {
				if (!child[n].href)
					child[n].href = this.dummyHref;
			}

			if (child[n].childNodes)
				this.makeLinksTabbable(child[n]);
		}
	},

	removeFocusabilityToLinks : function() {
		var childs = document.getElementsByTagName('a');
		for ( var n = 0; n < childs.length; n++) {
			if (childs[n].href) {
				childs[n].oldhref = childs[n].href;
				childs[n].href = null;
			}
		}
	},

	restoreFocusabilityToLinks : function() {
		var childs = document.getElementsByTagName('a');
		for ( var n = 0; n < childs.length; n++) {
			if (childs[n].oldhref) {
				childs[n].href = childs[n].oldhref;
				childs[n].oldhref = null;
			}
		}
	},

    gotoAnchor: function(name) {
		var childs = document.getElementsByTagName('a');
		for ( var n = 0; n < childs.length; n++) {
			if (childs[n].name && childs[n].name == name) {
                window.scroll(0, childs[n].offsetTop);
                return;
			}
		}
    },
    
	// single click event for the entire view
	onClick : function(e) {
		e.preventDefault();
		EPOC.Event.stop(e);
		this.clickHandler(EPOC.Event.getTarget(e));
	},

	// add a DOM element or a block of HTML to a given DOM element
	appendHTML : function(html, parent) {
        console.log("in appendHTML");
		var parent = EPOC.Dom.get(parent) || this.container;
		if (typeof html == 'string') {
			var node = document.createElement("div");
			node.innerHTML = html;
		} else {
			var node = html;
		}
		this.makeLinksTabbable(node);
		parent.appendChild(node);
        this.updateScrollbar();
	},

	appendHTMLString : function(htmlString, parent) {
		var parent = EPOC.Dom.get(parent);
        if(parent != undefined){
            parent.innerHTML = parent.innerHTML + htmlString;
            this.makeLinksTabbable(parent);
        }
        this.updateScrollbar();
	},

    // replace the inner contents of a node
    replaceHTML : function(html, id){
        var node = EPOC.Dom.get(id);
        if(node != undefined){
            node.innerHTML = html;
        } else {
            this.container.innerHTML = html;
        }
        this.updateScrollbar();
    },
    
	// recursive funciton which walks the DOM tree up from
	// a given node to find something to respond to, at this
	// time either a link or an expando
	clickHandler : function(node) {
        
        console.log("CLICK HANDLER");
        console.log(node.tagName);
        if(node.className){
            console.log(node.className);
        }
        if(node.id){
            console.log(node.id);
        }
		if (node === this.container
				|| (node.className && node.className.indexOf('content') >= 0)) {
            console.log("AT CONTAINER NO MORE CLICK PROCESSING");
			return;
		}

        if(node.id == "mask"){
            console.log("node is mask");
            bridge.handleBack();
        } else if(node.className && node.className.indexOf('return') >= 0) {
            console.log("return hit");
            bridge.handleBack();
		} else {
			var href = node.getAttribute('href');

			if (href && href != this.dummyHref) {
				// user tapped on a linkable element
                console.log("have an href node");
                var hilightParent = this.getHilightParent(node);
                if(hilightParent != undefined){
                    console.log("have a hilightParent");
                    EPOC.Dom.addCSSClass(hilightParent, "hilighted");
                    EPOC.Event.yield(
                        function(){
                            EPOC.Dom.removeCSSClass(hilightParent, "hilighted");
                            this.selectEvent.fire(href);
                        }, this, 200);
                } else {
                    this.selectEvent.fire(href);
                }
			} else {
				// continue searching up the DOM tree
                console.log("Pass on up to parent node");
				this.clickHandler(node.parentNode);
			}
		}
	},

    getHilightParent: function(node){
        var parent = node.parentNode;
        while(parent != undefined){
            if(parent.className && 
            parent.className.indexOf('hilightParent') >= 0) {
                return parent;
            }
            if(parent.className && parent.className.indexOf('content') >= 0) {
                return null;
            }
            parent = parent.parentNode;
            if (parent === this.container
				|| (parent.className && parent.className.indexOf('content') >= 0)) {
                return null;
            }
        }
        return null;
    },

	showPopup : function(content) {
        this.oldScrollPos = window.scrollY;
		this.mask.style.display = '';
		this.popup.style.display = '';
		this.removeFocusabilityToLinks();
		this.popupContent.innerHTML = content;
		window.scroll(0,0);
        EPOC.Event.yield(
            function(){
                if(this.popup.style.display != 'none'){
                    this.setupPopupDisplay(bridge.getViewHeight());
                    this.updateScrollbar();
                }
            }, this, 400);
	},

    setupPopupDisplay: function(height){
        var contentHeight = this.popup.offsetHeight + this.popup.offsetTop;
        if(contentHeight > height)
            height = contentHeight;        
        this.container.style.height = height + "px";
        this.container.style.setProperty("padding-bottom", "0px");
        this.container.style.overflow = 'hidden';
        this.mask.style.height = height + "px";
        this.mask.style.overflow = 'hidden';
        return height;
    },
    
	hidePopup : function() {
		this.mask.style.display = 'none';
		this.popup.style.display = 'none';
		this.restoreFocusabilityToLinks();
        this.container.style.setProperty("padding-bottom", "40px");
        this.container.style.height = 'auto';
        this.container.style.overflow = 'scroll';
        this.mask.style.height = 'auto';
        this.mask.style.overflow = 'scroll';
        setTimeout("window.scroll(0," + this.oldScrollPos + ")", 100);
        this.updateScrollbar();
	},
    
    onSizeChanged: function(width, height) {
        var diff = false;
        if(this.lastWidth > width + 20 || this.lastWidth < width - 20){
            diff = true;
        }
        if(this.lastHeight > height + 20 || this.lastHeight < height - 20){
            diff = true;
        }
        this.lastWidth = width;
        this.lastHeight = height;
        if(diff){
            var height = bridge.getViewHeight();
            var contentHeight = this.container.offsetHeight;
            if(this.popup.style.display != "none") {
                contentHeight = this.setupPopupDisplay(height);
            }
            this.updateScrollbarVisible(height, contentHeight);
        }
    },
    
    updateScrollbar: function(){
        var height = bridge.getViewHeight();
        var contentHeight = this.container.offsetHeight;
        this.updateScrollbarVisible(height, contentHeight);
    },
    
    updateScrollbarVisible: function(height, contentHeight){
        var showScroll = true;
        if(contentHeight <= height)
            showScroll = false;
        bridge.showScrollbar(showScroll);
    },
    
    updateImages: function(names, sources, count, delay){
        console.log("updateImages count:" + count);
        EPOC.Event.yield(
            function(){
                var i = 0;
                while(i < count){
                    this.updateImageSrc(names[i], sources[i]);
                    i = i + 1;
                }
            }, this, delay);
    },
    
    updateImageSrc: function(itemName, source){
        var item = EPOC.Dom.get(itemName);
        if(item != undefined){
            item.src = source;
        }
    }
    
});
