/*
	client.js -- Epocrates Mobile Detail Presenter Client
*/

// establish our namespace if not already defined
if (typeof EPOCRATES == "undefined")
	EPOCRATES = {};
	
/*
	EPOCRATES.md -- Mobile Detail module	
*/
EPOCRATES.md = (function(){
	// private variables
	
	var stack = {};
	var startSlide = 0;
	var storedSlide = 0;
	var currentSlide = 0;
	var domPage = document.getElementById('page');
	var tracking = null;
	var util = null;
	var mdetail_id = 0;
	var questionsLeft = 0;
	var answers = new Array();
	var essPointsEnv = "";
	var importantInfo = "Important Info";
	var drugID = "0";
	var oldStyle = true;

	// public methods
	return {
		// called on document load, entry point
		init: function() {
			tracking = EPOCRATES.md.tracking;
			util = EPOCRATES.md.util;

			if (typeof EPOCRATES.md.getEnv != "undefined" && EPOCRATES.md.getEnv() != 'unknown') {
				essPointsEnv = EPOCRATES.md.getEnv();
			}

			if (typeof EPOCRATES.md.getImportantInfoText != "undefined" && EPOCRATES.md.getImportantInfoText() != 'unknown') {
				importantInfo = EPOCRATES.md.getImportantInfoText();
			}

			if (typeof EPOCRATES.md.getDrugIDText != "undefined" && EPOCRATES.md.getDrugIDText() != 'unknown') {
				drugID = EPOCRATES.md.getDrugIDText();
			}

		},
		
		// read our stack, used in JSON loading callback
		load: function(jsonData) {
			stack = jsonData;

            var slide = this.getStartSlide();

			//alert("stack.content_type_version = " + stack.content_type_version);
			if (stack.content_type_version == "2") {
				oldStyle = false;
			} else {
				oldStyle = true;
			}

            // record event for user tracking
            // Assumption : Begin button will always be in slide one
            if ((slide == 0) || (slide == 1 && !oldStyle)) {
            		// track launch
            		this.trackEvent(tracking.msg.LAUNCH, {
						"mdetail_status": "1",
						"slide_pos": slide
					});
            } else if (slide == 1 && oldStyle) {
					this.trackEvent(tracking.msg.LAUNCH, {
						"mdetail_status": "2",
						"slide_pos": slide
					});
            } else if (slide > 1) {
					this.trackEvent(tracking.msg.LAUNCH, {
						"mdetail_status": "2",
						"slide_pos": slide
					});
            }
			this.next(slide, true);
		},

		recordVideoLaunch: function() {
			var slide = currentSlide;
			this.trackEvent(tracking.msg.VIDEO_LAUNCH, {
						"mdetail_status": "1",
						"slide_id": stack.slides[slide].id,
						"slide_pos": slide                      
				});
		},
		
		getStartSlide: function() {
			//alert("getStartSlide started");
            // it appears that the # in <a href="#" ..> will be included in some urls, e.g. /nothankyou
            // therefore we need to get rid of it here.
            var urlnoanchor = window.location.href.split("#")[0];
			//var url = window.location.href.split("?");
            var url = urlnoanchor.split("?")[1];
			var startSlide = 0;
			var lastSlide = 0;

			// save mdetailid passed in url for use in tracking
			if (typeof url != 'undefined' && url.length > 1) {
				if (url.substring(0, 10) == "mdetailid=") {					
					//  The following is assuming that there is nothing else
					//  except for the mdetail id.
					mdetail_id = url.substring(10);
				}					
			}

            // check our persistent data sotrage for last slide
            //var lastSlide = storage.get(mdetail_id);
            lastSlide =  this.storedSlide;
				
            //console.log(lastSlide);

            if (lastSlide)
					startSlide = lastSlide;
                    
			startSlide = parseInt(startSlide);
			
			//console.log(startSlide);

			return (isNaN(startSlide)) ? 0 : startSlide;		
		},

		setStoredSlide: function(slide) {
			this.storedSlide = slide;
			//alert("setStoredSlide="+slide);
		},

		// next slide
		next: function(slide, launch) {
			if (typeof slide == 'undefined')
				var slide = currentSlide + 1;
				
			if (slide >= 0 && slide < stack.slides.length) {
				currentSlide = slide;
				this.setPageContents(stack.slides[slide].content, slide, stack.slides.length);

				questionsLeft = (stack.slides[slide].content.split('\<dl ').length)-1;				
				//console.log("questionsLeft="+questionsLeft);
				this.setButtonVisibility(questionsLeft);
                                
                if(launch != true){
				   // record event for user tracking
                   // Assumption : Begin button will always be in slide one     
				  	if ((slide == 1 && oldStyle) || (slide == 2 && !oldStyle)) { 
                      	this.trackEvent(tracking.msg.BEGIN, {
							"mdetail_status": "2",
							"slide_pos": slide						
						});
                    } else if (slide == 2 && oldStyle) {
						this.trackEvent(tracking.msg.SLIDE_CHANGE, {
                    		"mdetail_status": "2",
                    		"slide_id": stack.slides[slide - 1].id,
                    		"slide_pos": slide
						});
					} else if (slide > 2) {
						this.trackEvent(tracking.msg.SLIDE_CHANGE, {
                    		"mdetail_status": "2",
                    		"slide_id": stack.slides[slide - 1].id,
                    		"slide_pos": slide
						});
				    }
				}
			}
			
			this.storedSlide = slide;
		},

		setButtonVisibility: function(questionsLeft) {
			var nextbutton = document.getElementById('nextbutton');
			if (typeof nextbutton != 'undefined' && nextbutton != null) {
				if (questionsLeft <= 0) {
					nextbutton.style.visibility = 'visible';
				} else {
					nextbutton.style.visibility = 'hidden';
				}
			}
		},

		// load our page with HTML
		setPageContents: function(html, currentPage, totalPages) {
			if (totalPages > 0) {
				//var appendText = '<p>&nbsp;</p><p style="text-align:center;">' + (currentPage + 1) + ' of ' + totalPages + '</p><br/>';
				var appendText = '';
				if (essPointsEnv == 'dev' || essPointsEnv == 'qa') {
					appendText += '<a style="text-decoration:none;" href="javascript:window.print();">.</a><br/>';
				}
				domPage.innerHTML = html + appendText;
			} else {
				domPage.innerHTML = html;
			}
			
			// move our scroll position back to the top
			window.scrollTo(0, 0);
		},
		
		// toggle an expando widget
		expando: function(element) {
			if (typeof element.reveal == 'undefined')
				element.reveal = util.getChildNodeByType(element, 'CITE');

			// toggle the classname, let CSS define how it opens
			if (typeof element.reveal != 'undefined')
			{
				if (element.reveal.className == 'open') {
					element.reveal.className = 'null';
					element.style.backgroundImage = 'url(css/images/down-arrow.png)';
				} else {
					element.reveal.className = 'open';
					element.style.backgroundImage = 'url(css/images/up-arrow.png)';
				}
			}
				//element.reveal.className = (element.reveal.className == 'open') ? '' : 'open';
		},

		displayAnswers: function(question, anyanswer) {
			var questionId = question.id;

			if (typeof answers != 'undefined' && answers[questionId] != null) {
				// loop around answers to find a match with questionId.
				// if there is a match, then display the match
				// then stop because a match was found.
				var prevAnswer = answers[questionId];
				var options = question.getElementsByTagName('dt');

				for (var x=0; x<options.length; x++) {
					var dtTag = options[x];
					if (dtTag.id == prevAnswer) {
						//alert("dtTag.id needing to be open => " + dtTag.id);

						var ddMatch = dtTag.nextSibling;
						while (ddMatch) {
							if (ddMatch.tagName && ddMatch.tagName.toUpperCase() == 'DD') {
								break; /*this is a DD item. you're good.*/
							}
							ddMatch = ddMatch.nextSibling;
						}

						ddMatch.className='open';
						
						//console.log("dtMatch.id: " + dtMatch.id);
						if (typeof anyanswer === 'undefined' || anyanswer != 'true') {
							questionsLeft = questionsLeft - 1;
							this.setButtonVisibility(questionsLeft);
						}
					}
				}
			}
		},

		storeCombo: function(question, answer) {
			//alert("answers[question.id]="+answers[question.id]+ " and answer.id="+answer.id);
			if (typeof answers[question.id]!='undefined') {
				return true;
			}
			if (typeof question != 'undefined') {
				//The answer can exist or not.
				//If it does, store answer to this question.
				//otherwise, just display the stored answer if any.
				if (typeof answer != 'undefined') {
					//Store the q/a combo to be used elsewhere.
					answers[question.id] = answer.id;
				}
				this.displayAnswers(question);
			}
			return false;
		},

		// select an answer and record the event
		answer: function(element, anyanswer) {
			var question = element.parentNode;
			var questionId = question.id;
			var answerId = element.id;

			//alert("answerId :" + answerId);
			if (typeof anyanswer !== 'undefined') {
				//this.storeCombo(question,element)
				if (this.storeCombo(question,element)) {
					answers[question.id] = element.id;
					this.displayAnswers(question,'true');
					return;
				} else {
					//console.log("first time answering this");
				}
			} else {
				if (this.storeCombo(question,element)) {
					//console.log("you already answered this question");
					return;
				}
			}

			// record event for user tracking
			this.trackEvent(tracking.msg.ANSWER, {
				"question_id": questionId,
				"answer_id": answerId
			});
		},
		
		// select or unselect a given answer and its expando bit
		setAnswerState: function(element, state) {
			var reveal = element.nextSibling;
			reveal.className = state ? 'open' : '';
			element.className = reveal.className;
		},
		
		// load our stack, and optionally move to a slide index
		// if called with index.html?n where n is an integer referring to slide index
		loadStack: function() {
			// load our JSON data stack
			util.loadScript("data/detail.json");
		},
		
		lookupSlideById: function(id) {
			for (var i = 0, l = stack.slides.length; i < l; i++) {
				if (typeof stack.slides[i].id != 'undefined' && stack.slides[i].id == id) {
					return i;
				}
			}

			// lookup didn't work, start at the beginning
			return 0;
		},

		showDrugID: function() {
			return drugID;
		},

		showToolbar: function() {
			return importantInfo;
		},

		// display a named message, usually activity or info
		showMessage: function(name) {
			if (name == "info") {
				this.setPageContents(stack.messages[0].content, 0, 0);
			} else {
				this.setPageContents(stack.slides[currentSlide].content, currentSlide, stack.slides.length);
				var arr = stack.slides[currentSlide].content.split('\<dl ');
				questionsLeft = arr.length-1;

				for (var i=0; i<arr.length; i++) {
					//console.log("split ["+i+"] = " + arr[i]);
					if (i>0) {
						var questionId = arr[i].substr(4,36);
						//console.log("questionId = '" + questionId + "'");
						var question = document.getElementById(questionId);
						this.displayAnswers(question);
					}
				}

				this.setButtonVisibility(questionsLeft);
			}
		},
		
		moreinfo: function(index) {
			this.setPageContents(stack.messages[index].content, 0, 0);
		},

		// notify the client app that the user did something significant
		trackEvent: function(eventID, data) {            
			if (typeof data == 'undefined')
				var data = {};
				
			data.mdetail_id = mdetail_id;
			
			// get our JSON strong from our data

			tracking.send("/tracking?eventID=" + eventID + "&trackingData=" + escape(util.toJSONString(data)));
		},
		
		// this takes us back to the native app in iPhone
		done: function() {
			this.setPageContents("", 0, 0);
			tracking.send("/go_to_list");
		},

		optin: function() {
			//ORIG: tracking.send("/optin?status=1");
			this.trackEvent(tracking.msg.OPT_IN, {
						"optin_status": "1",
						"mdetail_status": "3"
						});
		},

		optout: function() {
			//ORIG: tracking.send("/optin?status=2");
			this.trackEvent(tracking.msg.OPT_IN, {
						"optin_status": "2",
						"mdetail_status": "3"
						});
			
		},

		optinUpfront: function() {
			var slide = this.currentSlide;
			//ORIG: tracking.send("/optinfront?status=1");
			this.trackEvent(tracking.msg.OPT_IN_FRONT, {
						"optin_status": "1"
						});
			this.next(slide, true);
		},

		optoutUpfront: function() {
            var slide = this.currentSlide;
			//ORIG: tracking.send("/optinfront?status=2");
			this.trackEvent(tracking.msg.OPT_IN_FRONT, {
						"optin_status": "2"
						});
        	this.next(slide, true);
		},

		upfrontDone: function() {
			//ORIG: tracking.send("/optinfrontdone");
			this.trackEvent(tracking.msg.OPT_IN_DONE, {
						"mdetail_status": "3"
						});
		},

		nothankyou: function() {
			tracking.send("/nothankyou");
		},

		insertPdf: function() {
			var Event = YAHOO.util.Event;
			var myEditor = window.opener.YAHOO.widget.EditorInfo.getEditorById('edit');
			//Get a reference to the editor on the other page

			//Add a listener to the parent of the images
			Event.on(document.getElementById('pdfstable'), 'click', function(ev) {
				var tar = Event.getTarget(ev);
				//Check to see if we clicked on an image
				if (tar && tar.tagName && (tar.tagName.toLowerCase() == 'a')) {
					//Focus the editor's window
					myEditor._focusWindow();
					//Fire the execCommand for insertimage
					var str = '<a href=' + tar.getAttribute('href') + '>' +
						tar.getAttribute('href') + '</a>';
					myEditor.execCommand('inserthtml', str);
					//Close this window
					window.close();
				}
			});
		},

		insertImage: function() {
			var Event = YAHOO.util.Event;
			var myEditor = window.opener.YAHOO.widget.EditorInfo.getEditorById('edit');
			//Get a reference to the editor on the other page

			//Add a listener to the parent of the images
			Event.on(document.getElementById('imagestableitem'), 'click', function(ev) {
				var tar = Event.getTarget(ev);
				//Check to see if we clicked on an image
				if (tar && tar.tagName && (tar.tagName.toLowerCase() == 'img')) {
					//Focus the editor's window
					myEditor._focusWindow();

					//Fire the execCommand for insertimage
					if (tar.width < 320) {
						var str = '<img style="width:+' + tar.width + 'px;" src=' + tar.getAttribute('src',2) + '>';
						myEditor.execCommand('inserthtml', str);
					} else {
						myEditor.execCommand('insertimage', tar.getAttribute('src',2));
					}

					window.close();
				}
			});
		},

		// This function is now deprecated.  There may still be a call from the editor,
		// but it is not doing anything.
		lockdown: function(q) {
			//q.alreadyAnswered = true;
		}
	}
})();


