/*
	storage.js -- MDetail persistent data storage, key/value based
	
	Platform: for WebView with no localStorage support, but with basic HTML5 SQL
*/

EPOCRATES.md.storage = (function(){
	var db;
	var data = {};
	var initCallback = function() {};
	var initContext = {};
	
	function connect(databaseName) {
		db = openDatabase(databaseName, "1.0", databaseName, 65535);
		getData();
	}
	
	function getData() {
		db.transaction(
			function(transaction) {
				transaction.executeSql("select id, value from storage", [], function(transaction, result) {
					console.log("got " + result.rows.length + " items");
					for (var i = 0, l = result.rows.length; i < l; i++) {
						var item = result.rows.item(i);
						data[item.id] = item.value;
					}
					doneHandler();
				}, createTable);
			}
		);
	}
	
	function createTable() {
		db.transaction(
			function(transaction) {
				transaction.executeSql("create table storage (id varchar not null primary key, value varchar not null)", [], doneHandler, doneHandler);
			}
		);
	}
	
	function errorHandler(error) {
		console.log(error);
	}
	
	function successHandler() {
		// NOP
	}
	
	function doneHandler() {
		initCallback.call(initContext);
	}
	
	return {
		// expects the GUID for the slide deck
		init: function(callback, context) {
			initCallback = callback;
			initContext = context;
			
			if (typeof openDatabase == 'undefined') {
				callback.call(context);
			}
			else {
				connect('detail');
			}
		},
		
		done: function() {
			this.callback.call(context);
		},
		
		set: function(id, value) {
			data[id] = value;
			
			if (typeof openDatabase == 'undefined') {
				return;
			}

			db.transaction(
				function(transaction) {
					transaction.executeSql("insert into storage (id, value) values (?, ?)", [id, value], successHandler, function() {
						transaction.executeSql("update storage set value = ? where id = ?", [value, id], successHandler, errorHandler)
					})
					
				}
			);
		},
	
		get: function(id) {
			return data[id];
		},
		
		cleanup: function() {
			// Stubbed out for iPhone, not needed
		}
	}
})();
